==> en-select-1.0.1.tgz: package/addon/components/en-select.js <==
import Component from '@ember/component'
import { A } from '@ember/array'
import { run } from '@ember/runloop'
import { isEmpty } from '@ember/utils'
import {
  computed,
  set,
  get,
  getWithDefault,
  getProperties,
} from '@ember/object'

import KeyMap from 'en-select/utils/keymap'
import basicSearch from 'en-select/utils/basic-search'
import { fallback } from 'en-select/utils/fallback'

export default Component.extend({
  classNames: ['en-select'],
  classNameBindings: [
    'isOpen:is-open',
    'isFixed:is-fixed',
    'isDisabled:is-disabled',
    'hasSelections:has-selections',
  ],

  attributeBindings: ['tabindex', 'role'],
  documentEvents: ['click', 'resize', 'scroll'],

  tabindex: 0,
  role: 'listbox',

  multiple: false,

  canRemoveSelection: true,

  optionLabelPath: 'label',
  optionValuePath: 'id',

  isOpen: false,
  isDisabled: false,
  isClosing: false,
  isFixed: false,

  positionFromTop: false,
  positionFromRight: false,

  options: fallback(A([])),
  selections: fallback(A([])),
  focused: null,

  search: null,
  canSearch: true,

  label: 'Search...',

  init() {
    this._super(...arguments)

    let { options, selections } = getProperties(this, 'options', 'selections')

    if (!options || isEmpty(options)) {
      set(this, 'options', A([]))
    }

    if (!selections || isEmpty(selections)) {
      set(this, 'selections', A([]))
    }

    this._syncSelectionsWithValue()
  },

  didReceiveAttrs() {
    this._syncSelectionsWithValue()
  },

  keyUp(e) {
    const { which, keyCode } = e
    const key = which || keyCode

    const isOpen = get(this, 'isOpen')

    switch(key) {
      case KeyMap.enter:
      case KeyMap.down:
        if (!isOpen) this.openSelect()
        break;

      case KeyMap.esc:
        if (isOpen) this.closeSelect()
        break;

      default:
        break;
    }
  },

  hasSelections: computed.gt('selections.length', 0),

  selection: computed('selections.[]', {
    get() {
      return get(this, 'selections.firstObject')
    },
  }),

  optionsThatMatchSearch: computed('search', 'options.[]', {
    get() {
      const { options, search } = getProperties(this, 'options', 'search')
      const labelPath = getWithDefault(this, 'optionLabelPath', 'label')

      if (isEmpty(search)) {
        return options
      } else {
        return basicSearch(search, options, { labelPath })
      }
    }
  }),

  optionsAvailableToSelect: computed.setDiff('optionsThatMatchSearch', 'selections'),

  onDocumentClick(e) {
    const { isOpen, multiple, selectableCount } = getProperties(
      this,
      'isOpen',
      'multiple',
      'selectableCount',
    )

    if (!isOpen) {
      return false
    }

    /*
     * If it's an outside click, we close in all cases.
    */

    if (!this.element.contains(e.target)) {
      this.closeSelect()
    }

    /*
     * If it's an inside click (e.g. an option), we see
     * if it's a multiple select and return if it is.
    */

    if (multiple && selectableCount > 0) {
      return
    }

    /* Otherwise, we're safe to close. */
    this.closeSelect()
  },

  /**
   * openSelect
   *
   * Opens the dropdown and focuses on the search box
   */
  openSelect() {
    if (get(this, 'isDisabled')) return

    run(() => {
      set(this, 'isOpen', true)
      this._focusOnSearch()
    })
  },

  /**
   * closeSelect
   *
   * Closes the dropdown
   */

  closeSelect() {
    run(() => {
      if (get(this, 'isDestroyed')) return
      set(this, 'isOpen', false)
    })
  },

  /**
   * focusOption
   *
   * Sets the focus to a given option and scrolls to it
   * if it's not visible.
   */

  focusOption(option, event) {
    set(this, 'focused', option)

    if (event && event instanceof KeyboardEvent) {
      this._scrollIfFocusNotVisible()
    }

    if (this.attrs['on-focus']) {
      this.attrs['on-focus'](option)
    }
  },

  /**
   * selectOption
   *
   * Selects a given option. There are a couple of scenarios
   * with different behaviors:
   *
   * (a) when it's a multi-select, we select the option, fire
   * an action, and set the focus on search (i.e. don't close
   * the dropdown)
   *
   * (b) when it's a single-select, select the option and close
   * the dropdown.
   *
   * Finally, set the search and focus to null.
   */

  selectOption(option, event) {
    event.stopPropagation()

    let multiple = get(this, 'multiple')
    let selections = get(this, 'selections')

    this._select(option)

    if (multiple) {
      this.attrs['on-select'](selections)
      this._focusOnSearch()

    } else {
      this.attrs['on-select'](get(this, 'selection'))
      this.closeSelect()
      this.element.focus()
    }

    set(this, 'search', null)
    set(this, 'focused', null)
  },

  /**
   * removeSelection
   *
   * Removes a given option and fires an action.
   */

  removeSelection(option = null) {
    const multiple = get(this, 'multiple')

    if (multiple) {
      get(this, 'selections').removeObject(option)
    } else {
      set(this, 'selections', A([]))
    }

    if (this.attrs['on-remove']) {
      this.attrs['on-remove'](option)
    }
  },

  /**
   * @private
   * _syncSelectionsWithValues()
   *
   * Takes the value supplied to the component and sets the selections from it.
   */

  _syncSelectionsWithValue() {
    let value = get(this, 'value')
    let multiple = get(this, 'multiple')

    if (!value) return

    if (multiple) {
      value.forEach(value => this._select(value))
    } else {
      this._select(value)
    }
  },

  _select(option) {
    if (get(this, "isDestroyed")) return

    let { options, multiple, optionValuePath } = getProperties(
      this, "options", "multiple", "optionValuePath"
    )

    if (isEmpty(options) || isEmpty(option)) return

    let id

    /*
     * Figure out the ID of the object. If the value
     * we get is a string, we just match the IDs to
     * be sure.
     *
     * In case it's an object, we find the ID using the
     * `optionValuePath` and use that to find the selected
     * option.
    */

    if (typeof option === "object") {
      id = get(option, optionValuePath)
    } else {
      id = option
    }

    let optionsThatMatch = options.filter(option => {
      return get(option, optionValuePath) === id
    })

    if (multiple) {
      get(this, "selections").addObjects(optionsThatMatch)
    } else {
      set(this, "selections", A(optionsThatMatch))
    }
  },

  _focusOnSearch() {
    run.scheduleOnce('afterRender', this, () => {
      const searchBox = this.element.querySelector('.en-select-search')
      if (searchBox) searchBox.focus()
    })
  },

  _scrollIfFocusNotVisible() {
    let node = this.element
    if (!node || !(node instanceof HTMLElement)) return

    let focusedOption = node.querySelector('.en-select-option.is-focused')

    if (!focusedOption) {
      return
    }

    return focusedOption.scrollIntoView()
  },
})

==> en-select-1.0.2.tgz: package/addon/components/en-select.js <==
import Component from '@ember/component'
import { A } from '@ember/array'
import { run } from '@ember/runloop'
import { isEmpty } from '@ember/utils'
import {
  computed,
  set,
  get,
  getWithDefault,
  getProperties,
} from '@ember/object'

import KeyMap from 'en-select/utils/keymap'
import basicSearch from 'en-select/utils/basic-search'
import { fallback } from 'en-select/utils/fallback'

export default Component.extend({
  classNames: ['en-select'],
  classNameBindings: [
    'isOpen:is-open',
    'isFixed:is-fixed',
    'isDisabled:is-disabled',
    'hasSelections:has-selections',
  ],

  attributeBindings: ['tabindex', 'role'],
  documentEvents: ['click', 'resize', 'scroll'],

  tabindex: 0,
  role: 'listbox',

  multiple: false,

  canRemoveSelection: true,

  optionLabelPath: 'label',
  optionValuePath: 'id',

  isOpen: false,
  isDisabled: false,
  isClosing: false,
  isFixed: false,

  positionFromTop: false,
  positionFromRight: false,

  options: fallback(A([])),
  selections: fallback(A([])),
  focused: null,

  search: null,
  canSearch: true,

  label: 'Search...',

  init() {
    this._super(...arguments)

    let { options, selections } = getProperties(this, 'options', 'selections')

    if (!options || isEmpty(options)) {
      set(this, 'options', A([]))
    }

    if (!selections || isEmpty(selections)) {
      set(this, 'selections', A([]))
    }

    this._syncSelectionsWithValue()
  },

  didReceiveAttrs() {
    this._syncSelectionsWithValue()
  },

  keyUp(e) {
    const { which, keyCode } = e
    const key = which || keyCode

    const isOpen = get(this, 'isOpen')

    switch(key) {
      case KeyMap.enter:
      case KeyMap.down:
        if (!isOpen) this.openSelect()
        break;

      case KeyMap.esc:
        if (isOpen) this.closeSelect()
        break;

      default:
        break;
    }
  },

  hasSelections: computed.gt('selections.length', 0),

  selection: computed('selections.[]', {
    get() {
      return get(this, 'selections.firstObject')
    },
  }),

  optionsThatMatchSearch: computed('search', 'options.[]', {
    get() {
      const { options, search } = getProperties(this, 'options', 'search')
      const labelPath = getWithDefault(this, 'optionLabelPath', 'label')

      if (isEmpty(search)) {
        return options
      } else {
        return basicSearch(search, options, { labelPath })
      }
    }
  }),

  optionsAvailableToSelect: computed.setDiff('optionsThatMatchSearch', 'selections'),

  onDocumentClick(e) {
    const { isOpen, multiple, selectableCount } = getProperties(
      this,
      'isOpen',
      'multiple',
      'selectableCount',
    )

    if (!isOpen) {
      return false
    }

    /*
     * If it's an outside click, we close in all cases.
    */

    if (!this.element.contains(e.target)) {
      this.closeSelect()
    }

    /*
     * If it's an inside click (e.g. an option), we see
     * if it's a multiple select and return if it is.
    */

    if (multiple && selectableCount > 0) {
      return
    }

    /* Otherwise, we're safe to close. */
    this.closeSelect()
  },

  /**
   * openSelect
   *
   * Opens the dropdown and focuses on the search box
   */
  openSelect() {
    if (get(this, 'isDisabled')) return

    run(() => {
      set(this, 'isOpen', true)
      this._focusOnSearch()
    })
  },

  /**
   * closeSelect
   *
   * Closes the dropdown
   */

  closeSelect() {
    run(() => {
      if (get(this, 'isDestroyed')) return
      set(this, 'isOpen', false)
    })
  },

  /**
   * focusOption
   *
   * Sets the focus to a given option and scrolls to it
   * if it's not visible.
   */

  focusOption(option, event) {
    set(this, 'focused', option)

    if (event && event instanceof KeyboardEvent) {
      this._scrollIfFocusNotVisible()
    }

    if (this.attrs['on-focus']) {
      this.attrs['on-focus'](option)
    }
  },

  /**
   * selectOption
   *
   * Selects a given option. There are a couple of scenarios
   * with different behaviors:
   *
   * (a) when it's a multi-select, we select the option, fire
   * an action, and set the focus on search (i.e. don't close
   * the dropdown)
   *
   * (b) when it's a single-select, select the option and close
   * the dropdown.
   *
   * Finally, set the search and focus to null.
   */

  selectOption(option, event) {
    event.stopPropagation()

    let multiple = get(this, 'multiple')

    this._select(option)
    this.attrs['on-select'](option)

    if (multiple) {
      this._focusOnSearch()

    } else {
      this.closeSelect()
      this.element.focus()
    }

    set(this, 'search', null)
    set(this, 'focused', null)
  },

  /**
   * removeSelection
   *
   * Removes a given option and fires an action.
   */

  removeSelection(option = null) {
    const multiple = get(this, 'multiple')

    if (multiple) {
      get(this, 'selections').removeObject(option)
    } else {
      set(this, 'selections', A([]))
    }

    if (this.attrs['on-remove']) {
      this.attrs['on-remove'](option)
    }
  },

  /**
   * @private
   * _syncSelectionsWithValues()
   *
   * Takes the value supplied to the component and sets the selections from it.
   */

  _syncSelectionsWithValue() {
    let value = get(this, 'value')
    let multiple = get(this, 'multiple')

    if (!value) return

    if (multiple) {
      value.forEach(value => this._select(value))
    } else {
      this._select(value)
    }
  },

  _select(option) {
    if (get(this, "isDestroyed")) return

    let { options, multiple, optionValuePath } = getProperties(
      this, "options", "multiple", "optionValuePath"
    )

    if (isEmpty(options) || isEmpty(option)) return

    let id

    /*
     * Figure out the ID of the object. If the value
     * we get is a string, we just match the IDs to
     * be sure.
     *
     * In case it's an object, we find the ID using the
     * `optionValuePath` and use that to find the selected
     * option.
    */

    if (typeof option === "object") {
      id = get(option, optionValuePath)
    } else {
      id = option
    }

    let optionsThatMatch = options.filter(option => {
      return get(option, optionValuePath) === id
    })

    if (multiple) {
      get(this, "selections").addObjects(optionsThatMatch)
    } else {
      set(this, "selections", A(optionsThatMatch))
    }
  },

  _focusOnSearch() {
    run.scheduleOnce('afterRender', this, () => {
      const searchBox = this.element.querySelector('.en-select-search')
      if (searchBox) searchBox.focus()
    })
  },

  _scrollIfFocusNotVisible() {
    let node = this.element
    if (!node || !(node instanceof HTMLElement)) return

    let focusedOption = node.querySelector('.en-select-option.is-focused')

    if (!focusedOption) {
      return
    }

    return focusedOption.scrollIntoView()
  },
})
